@@ -7,6 +7,7 @@ import stdlibs
 import sys
 import traceback
 import os
+import re
 from datetime import datetime
 from pathlib import Path
 from neo4j.exceptions import CypherSyntaxError
@@ -21,18 +22,10 @@ from .core.watcher import CodeWatcher
 from .tools.graph_builder import GraphBuilder
 from .tools.code_finder import CodeFinder
 from .tools.import_extractor import ImportExtractor
-
+from .utils.debug_log import debug_log
 
 logger = logging.getLogger(__name__)
 
-def debug_log(message):
-    """Write debug message to a file for development and testing."""
-    debug_file = os.path.expanduser("~/mcp_debug.log")
-    timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
-    with open(debug_file, "a") as f:
-        f.write(f"[{timestamp}] {message}\n")
-        f.flush()
-
 class MCPServer:
     """
     The main MCP Server class.
@@ -132,7 +125,7 @@ class MCPServer:
                     "properties": {
                         "query_type": {"type": "string", "description": "Type of relationship query to run.", "enum": ["find_callers", "find_callees", "find_all_callers", "find_all_callees", "find_importers", "who_modifies", "class_hierarchy", "overrides", "dead_code", "call_chain", "module_deps", "variable_scope", "find_complexity", "find_functions_by_argument", "find_functions_by_decorator"]},
                         "target": {"type": "string", "description": "The function, class, or module to analyze."},
-                        "context": {"type": "string", "description": "Optional: specific file path for precise results."}
+                        "context": {"type": "string", "description": "Optional: specific file path for precise results."} 
                     },
                     "required": ["query_type", "target"]
                 }
@@ -197,7 +190,7 @@ class MCPServer:
                     "type": "object",
                     "properties": {
                         "function_name": {"type": "string", "description": "The name of the function to analyze."},
-                        "file_path": {"type": "string", "description": "Optional: The full path to the file containing the function for a more specific query."}
+                        "file_path": {"type": "string", "description": "Optional: The full path to the file containing the function for a more specific query."} 
                     },
                     "required": ["function_name"]
                 }
@@ -307,12 +300,21 @@ class MCPServer:
             return {"error": "Cypher query cannot be empty."}
 
         # Safety Check: Prevent any write operations to the database.
+        # This check first removes all string literals and then checks for forbidden keywords.
         forbidden_keywords = ['CREATE', 'MERGE', 'DELETE', 'SET', 'REMOVE', 'DROP', 'CALL apoc']
-        query_upper = cypher_query.upper()
-        if any(keyword in query_upper for keyword in forbidden_keywords):
-            return {
-                "error": "This tool only supports read-only queries. Prohibited keywords like CREATE, MERGE, DELETE, SET, etc., are not allowed."
-            }
+        
+        # Regex to match single or double quoted strings, handling escaped quotes.
+        string_literal_pattern = r'"(?:\\.|[^"\\])*"|\'(?:\\.|[^\'\\])*\''
+        
+        # Remove all string literals from the query.
+        query_without_strings = re.sub(string_literal_pattern, '', cypher_query)
+        
+        # Now, check for forbidden keywords in the query without strings.
+        for keyword in forbidden_keywords:
+            if re.search(r'\b' + keyword + r'\b', query_without_strings, re.IGNORECASE):
+                return {
+                    "error": "This tool only supports read-only queries. Prohibited keywords like CREATE, MERGE, DELETE, SET, etc., are not allowed."
+                }
 
         try:
             debug_log(f"Executing Cypher query: {cypher_query}")
@@ -785,7 +787,7 @@ class MCPServer:
                     response = {
                         "jsonrpc": "2.0", "id": request_id,
                         "result": {
-                            "protocolVersion": "2024-11-05",
+                            "protocolVersion": "2025-03-26",
                             "serverInfo": {
                                 "name": "CodeGraphContext", "version": "0.1.0",
                                 "systemPrompt": LLM_SYSTEM_PROMPT
@@ -804,24 +806,40 @@ class MCPServer:
                     tool_name = params.get('name')
                     args = params.get('arguments', {})
                     result = await self.handle_tool_call(tool_name, args)
-                    response = {
-                        "jsonrpc": "2.0", "id": request_id,
-                        "result": {"content": [{"type": "text", "text": json.dumps(result, indent=2)}]}
-                    }
+                    
+                    if "error" in result:
+                        response = {
+                            "jsonrpc": "2.0", "id": request_id,
+                            "error": {"code": -32000, "message": "Tool execution error", "data": result}
+                        }
+                    else:
+                        response = {
+                            "jsonrpc": "2.0", "id": request_id,
+                            "result": {"content": [{"type": "text", "text": json.dumps(result, indent=2)}]}
+                        }
+                elif method == 'notifications/initialized':
+                    # This is a notification, no response needed.
+                    pass
                 else:
                     # Handle unknown methods.
-                    response = {
-                        "jsonrpc": "2.0", "id": request_id,
-                        "error": {"code": -32601, "message": f"Method not found: {method}"}
-                    }
+                    if request_id is not None:
+                        response = {
+                            "jsonrpc": "2.0", "id": request_id,
+                            "error": {"code": -32601, "message": f"Method not found: {method}"}
+                        }
                 
-                # Send the response to standard output.
-                print(json.dumps(response), flush=True)
+                # Send the response to standard output if it's not a notification.
+                if request_id is not None and response:
+                    print(json.dumps(response), flush=True)
 
             except Exception as e:
                 logger.error(f"Error processing request: {e}\n{traceback.format_exc()}")
+                request_id = "unknown"
+                if 'request' in locals() and isinstance(request, dict):
+                    request_id = request.get('id', "unknown")
+
                 error_response = {
-                    "jsonrpc": "2.0", "id": request.get('id') if 'request' in locals() else None,
+                    "jsonrpc": "2.0", "id": request_id,
                     "error": {"code": -32603, "message": f"Internal error: {str(e)}", "data": traceback.format_exc()}
                 }
                 print(json.dumps(error_response), flush=True)
