@@ -9,6 +9,7 @@ from datetime import datetime
 
 from ..core.database import DatabaseManager
 from ..core.jobs import JobManager, JobStatus
+from ..utils.debug_log import debug_log
 
 logger = logging.getLogger(__name__)
 
@@ -16,15 +17,6 @@ logger = logging.getLogger(__name__)
 # Set to 1 to enable, 0 to disable.
 debug_mode = 0
 
-def debug_log(message):
-    """Write debug message to a file"""
-    debug_file = os.path.expanduser("~/mcp_debug.log")
-    timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
-    with open(debug_file, "a") as f:
-        f.write(f"[{timestamp}] {message}\n")
-        f.flush()
-
-
 class CyclomaticComplexityVisitor(ast.NodeVisitor):
     """Calculates cyclomatic complexity for a given AST node."""
     def __init__(self):
@@ -985,16 +977,29 @@ class GraphBuilder:
                 """, path=path)
 
     def delete_repository_from_graph(self, repo_path: str):
-        """Deletes a repository and all its contents from the graph."""
+        """
+        Deletes a repository and all its contents from the graph, then cleans up
+        any orphaned Module nodes that are no longer referenced.
+        """
         repo_path_str = str(Path(repo_path).resolve())
         with self.driver.session() as session:
+            # Delete the repository and all its contained elements, including parameters
             session.run("""
                 MATCH (r:Repository {path: $path})
                 OPTIONAL MATCH (r)-[:CONTAINS*]->(e)
-                DETACH DELETE r, e
+                OPTIONAL MATCH (e)-[:HAS_PARAMETER]->(p)
+                DETACH DELETE r, e, p
             """, path=repo_path_str)
             logger.info(f"Deleted repository and its contents from graph: {repo_path_str}")
 
+            # Clean up orphaned Module nodes that are no longer imported by any file
+            session.run("""
+                MATCH (m:Module)
+                WHERE NOT ()-[:IMPORTS]->(m)
+                DETACH DELETE m
+            """)
+            logger.info("Cleaned up orphaned Module nodes.")
+
     def update_file_in_graph(self, file_path: Path, repo_path: Path, imports_map: dict):
         """
         Updates a single file's nodes in the graph and returns its new parsed data.
